#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <boost/format.hpp>

#include "print.hh"

template <class SingletonVectorType, class VectorType>
void print_evolution(Dune::BitSetVector<1> const &frictionalNodes,
                     SingletonVectorType const &state,
                     VectorType const &displacement,
                     Dune::VirtualFunction<double, double> const &function,
                     int run, double time, std::ostream &octave_writer) {
  // Find the first node that belongs to the frictional boundary
  for (size_t i = 0; i < frictionalNodes.size(); ++i) {
    if (!frictionalNodes[i][0])
      continue;

    double out;
    function.evaluate(time, out);
    std::cout << boost::format("s[%03d] = %+3e, %|20t|u[%03d] = %+3e") % run %
                     state[i] % run % displacement[i][0] << std::endl;
    octave_writer << state[i][0] << " " << displacement[i][0] * 1e6 << " "
                  << out << std::endl;
    break;
  }
}

#include "print_tmpl.cc"
